#
# pushare - P2P file sharing system
#
# Copyright (c) 2004 Satoshi Fukutomi <fuktommy@inter7.jp>
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
# 3. Neither the name of the University nor the names of its contributors
#    may be used to endorse or promote products derived from this software
#    without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
# OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
# OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
# SUCH DAMAGE.
#
# $Id: Util.pm,v 1.2 2004/06/05 02:04:07 fuktommy Exp $
#

package PuShare::Util;
use strict;
use PuShare::Config;

#
# configuration
#
my $lock = $PuShare::Config::lock;
my $PI   = 3.1415926535;

sub standby() {
	open LOCK, "> $lock" or die "$lock: open failed. $!";
}

sub lock(;$) {
	flock LOCK, 002;
}

sub unlock() {
	flock LOCK, 010;
}

sub addDist($$) {
	my($stamp, $id) = @_;
	my $distlist = $PuShare::Config::distlist;
	my $savedist = $PuShare::Config::savedist;
	my @dist = ();
	if (open IN, $distlist) {
		@dist = <IN>;
		close IN;
	}
	push @dist, "$stamp<>$id\n";
	local $_;
	lock();
	open OUT, "> $distlist" or die "$distlist: failed to open. $!";
	foreach (@dist) {
		local @_ = split /<>/, $_;
		if ($stamp - $_[0] <= $savedist * 60) {
			print OUT;
		}
	}
	close OUT;
	unlock();
}

sub delDist($$) {
	my($stamp, $id) = @_;
	my $distlist  = $PuShare::Config::distlist;
	my @dist = ();
	if (open IN, $distlist) {
		@dist = <IN>;
		close IN;
	}
	lock();
	open OUT, "> $distlist" or die "$distlist: failed to open. $!";
	foreach (@dist) {
		print OUT if ($_ ne "$stamp<>$id\n");
	}
	close OUT;
	unlock();
}

sub addIgnore($) {
	my($id) = @_;
	my $ignorelist = $PuShare::Config::ignorelist;
	lock();
	open OUT, ">> $ignorelist" or die "$ignorelist: failed to open. $!";
	print OUT "$id\n";
	close OUT;
	unlock();
}

#
# nomal random (Box-Muller)
#
sub normal() {
	my($r1, $r2) = (rand, rand);
	return sqrt(-2 * log($r1)) * sin(2 * $PI * $r2);
}

1;
